#!/usr/bin/perl
#
# Script: listerr.pl - List ODL validation errors on a volume
# Usage:  listerr.pl [-f] <lvtool report>
#
# Author: Joel Wilf
# Date: 6/30/99
#
# Description:
#   This script scans the output of lvtool and lists all the errors.
#   If the -f option is set, the output is sorted by file.  Otherwise, 
#   it is sorted by error type.
#
#   Note: Prior to running listerr.pl, lvtool must be run with the
#   -nw option set.
# End:
#
# Revisions:
#   1999-08-18, J. Wilf, Removed extension filtering (it's done by lvtool)
#   1999-08-19, J. Wilf, Added sorting by error
#   2003-04-03, M. Cayanan, Added capability to find and print out warning 
#		and informational messages too.
# End:


use Getopt::Std;
$| = 1;

# Process command line
getopts('fe');     # If -f is set, $opt_f == 1
$usage = "Usage: listerr.pl [-f] <lvtool report>";

# 04-10-03 MDC
# Get the exclude file if '-e' was specified.
if($opt_e) {
   $exclude_file = shift or die "Can't find logfile";
   
   #Copy the messages onto an array
   open(EXCLUDE, "<$exclude_file") or die "Can't open logfile";
   
   while($line = <EXCLUDE>) {
      chop ($line);
      push @msgs_to_ignore, $line;
   }
   
   close EXCLUDE;
   
   foreach $exclude_msg (@msgs_to_ignore) {
   	
      $exclude_msg =~ s/(%d)/(\.*)/g;
      $exclude_msg =~ s/(%s)/(\.*)/g;
   }
}


$report = shift or die "$usage\n"; 
die "Can\'t find $report\n" unless (-e $report);
die "Invalid $report\n" unless (-f $report);

# Scan for errors
print "Finding Label Validation Errors:\n";
print "================================\n\n";
open (REPORT,"<$report");
while ($line = <REPORT>) {

    # Get filename
    if (substr($line,0,1) eq '#') {
        $line =~ m/(\w\S+)\s/;  # Extract file name from line.
        $file_name = $1;
        $file_name =~ s|\w:||g; # On Windows, get rid of volume letter.
#        $file_name =~ s|\\||g;  # Get rid of \.
	$file_name =~ s/\\/\//g; # Replace all '\' with '/'
        $file_name =~ s|^/||;   # Get rid of leading /.
    }
    
    # Filter out results from bad file types
    if (scalar @extlist != 0) {
    	$bad_file_flag = 0;
    	$bad_file_flag = 1 if ($file_name !~ m/\./);
    	foreach $extension (@extlist) {
    	    $bad_file_flag = 1 if ($file_name =~ m/\.$extension/i);
    	}
    	next if ($bad_file_flag == 1);
    }
        
    # Get syntactic errors 
     if ($line =~ m/ERROR:   (.+)/) {
     	$error = $1;
     	@split_error = split ' -- ', $error;
     	$error_type = $split_error[1];
     	
     	# 04-07-03 MDC - This is used in case an ERROR message is found that 
        # does not contain a line number such as a duplicate GROUP names ERROR
        # message. 
        if($split_error[1] eq "") {
             $error_type = $split_error[0];
        }
    	
    	# Check to see if the error message is part of our list
    	# we want to ignore
    	$found = check_msg($error_type);
    	
    	if($found == 0) {
    	   push @syn_errs, "$file_name: $error";
    	   push @error_types, $error_type;
    	}
    }
    
    # Get unfound pointer errors
        if ($line =~ m/Error File not found: (\w.+)/) {
    	$error = $1;
    	push @pnt_errs, "  $file_name --> $error";
    }
    
    # 04-02-03 MDC - Get Informational messages
    if ($line =~ m/INFO:    (.+)/) {
    	$info = $1;
    	@split_info = split ' -- ', $info;
    	$info_type = $split_info[1];
    	push @syn_info, "$file_name: $info";
    	push @info_types, $info_type;
    }
    
    # 04-02-03 MDC - Get Warning messages
    if ($line =~ m/WARNING: (.+)/) {
    	$warning = $1;
    	if($warning !~ /Label reading complete/) {
    	    @split_warning = split ' -- ', $warning;
    	    $warning_type = $split_warning[1];
    
            # 04-16-03 MDC - This is used in case a WARNING message is found that 
            # does not contain a line number.
             
            if($split_warning[1] eq "") {
                $warning_type = $split_warning[0];
            }
    	    
    	    #Check to see if the message type is on our 
    	    #list to ignore
    	    $found = check_msg($warning_type);
    	    
    	    if($found == 0) {
    	       push @syn_warning, "$file_name: $warning";
    	       push @warning_types, $warning_type;
    	    }
        }
    }
}
close REPORT;

# Sort all arrays
@syn_errs = sort @syn_errs;
@error_types = sort @error_types;
@pnt_errs = sort @pnt_errs;

@syn_info = sort @syn_info;
@info_types = sort @info_types;
@syn_warning = sort @syn_warning;
@warning_types = sort @warning_types;


# Default sort is by error type
unless ($opt_f == 1) {
	
    # Make an array of the unique error types
    $prev_type = "";
    while ($type = shift @error_types) {
    	push @unique_error_types, $type if ($type ne $prev_type);
        $prev_type = $type;
    }
    
    # Find all files that contain each error type
    foreach $unique_type (@unique_error_types) {
        push @sorted_errors, $unique_type;
        foreach $err (@syn_errs) {
            @split_err = split ' -- ', $err;
            $err_type = $split_err[1];   
            
            if($split_err[1] eq "") {
    	       @split_err = split /:\s+/, $err;
    	       $err_type = $split_err[1];
    	    }
        	
            if ($err_type eq $unique_type) {
                $file_and_line = "  " . $split_err[0];
                push @sorted_errors, $file_and_line;
            }
        }
    }
    
    #04-02-03 MDC - Do the same for info and warning messages
    $prev_type = "";
    while ($type = shift @warning_types) {
    	push @unique_warning_types, $type if ($type ne $prev_type);
    	$prev_type = $type;
    }
    
    foreach $unique_type (@unique_warning_types) {
        push @sorted_warnings, $unique_type;
    	foreach $warn (@syn_warning) {
    	    @split_warn = split ' -- ', $warn;
    	    $warn_type = $split_warn[1];
    	    
    	    if($split_warn[1] eq "") {
    	       @split_warn = split /:\s+/, $warn;
    	       $warn_type = $split_warn[1];
    	    }
    	    if ($warn_type eq $unique_type) {
    	    	$file_and_line = "  " . $split_warn[0];
    	    	push @sorted_warnings, $file_and_line;
    	    }
    	}
    }
    
    $prev_type = "";
    while ($type = shift @info_types) {
    	push @unique_info_types, $type if ($type ne $prev_type);
    	$prev_type = $type;
    }
    
    foreach $unique_type (@unique_info_types) {
       push @sorted_info, $unique_type;
       foreach $inf (@syn_info) {
       	   @split_inf = split ' -- ', $inf;
       	   $inf_type = $split_inf[1];
       	   if ($inf_type eq $unique_type) {
       	   	$file_and_line = "  " . $split_inf[0];
       	   	push @sorted_info, $file_and_line;
       	   }
       }
    }
}

### Display results

# Display Syntax Errors - by error type (default)
unless ($opt_f == 1) {
    if (scalar @sorted_errors > 0) {
        print "Syntax Errors:\n\n";
        print join "\n", @sorted_errors;
        print "\n";
    } else {
        print "There are no ODL syntax errors.\n";
    }
    print "\n\n";
    
    #Display Warnings and Info messages
    
    if (scalar @sorted_warnings > 0) {
    	print "Warning messages:\n\n";
    	print join "\n", @sorted_warnings;
    	print "\n";
    } else {
    	print "There are no warning messages.\n";
    }
    print "\n\n";
    
    if (scalar @sorted_info > 0) {
    	print "Info messages:\n\n";
    	print join "\n", @sorted_info;
    	print "\n";
    } else {
    	print "There are no info messages.\n";
    }
    print "\n\n";
}

# Display Syntax Errors - by file (-f option)
if ($opt_f == 1) {
    if (scalar @syn_errs > 0) {
        print "Syntax Errors:\n\n";
        $prev_entry = "";
        while ($entry = shift @syn_errs) {
            print "$entry\n" unless ($entry eq $prev_entry);
            $prev_entry = $entry;
        }
    } else {
        print "There are no ODL syntax errors\n";
    }
    print "\n\n";
    
    #04-02-03 MDC - Display info and warning messages too
    
    if (scalar @syn_warning > 0) {
    	print "Warning messages:\n\n";
    	$prev_entry = "";
    	while ($entry = shift @syn_warning) {
            print "$entry\n" unless ($entry eq $prev_entry);
            $prev_entry = $entry;
        }
    } else {
    	print "There are no warning messages\n";
    }
    print "\n\n";
    
    if (scalar @sys_info > 0) {
    	print "Info messages:\n\n";
    	$prev_entry = "";
    	while ($entry = shift @syn_info) {
    	    print "$entry\n" unless ($entry eq $prev_entry);
    	    $prev_entry = $entry;
    	}
    } else {
    	print "There are no info messages\n";
    }
    print "\n\n";
}

# Display Pointer Errors
@pnt_errs = sort @pnt_errs;
if (scalar @pnt_errs > 0) {
    print "Pointer Errors (file --> object pointed to but not found):\n\n";
    $prev_entry = "";
    while ($entry = shift @pnt_errs) {
        print "$entry\n" unless ($entry eq $prev_entry);
        $prev_entry = $entry;
    }
} else {
    print "There are no pointer errors.\n";
}
print "\n\n";


sub check_msg {
   my $msg = shift(@_);
   
   foreach $exclude_msg (@msgs_to_ignore) {
      if($msg =~ /(.*)$exclude_msg(\w)*/) {
         return 1;
      }
   }

   return 0;
   
}