#!/usr/bin/perl
#
# Script: tkvalid.pl - perl/tk front end to the PDS Validation Tools
# Usage:  tkvalid.pl [-d]
#
# Author: Joel Wilf
# Date: 08-29-99
#
# Description:
#   This script is a perl/tk front end the PDS Validataion Tools.
#   It gets user input through dialog windows and runs the appropriate
#   validation tests automatically. If the -d option is set, it will
#   run in debug mode and only print out the volume root and tests
#   selected. NOTE: tkvalid.pl requires that the user's perl 
#   installation include the Tk module.
# End:
#
# Revisions:
#   2000-04-03, J. Wilf, Added ISOCHECK and KWVTOOL checks.
#   2002-03-11, J. Wilf, Changed default directory to current one
#   2003-04-29, M. Cayanan, Added another button called "Change Label
#     Validation Settings" to allow user to modify certain parameters
#     if the Label Validation test is run. Also took out any reference
#     to checklab test since lvtool already does this through the 
#     label validation test.
#
# End:


use Tk;
use Cwd;
use Getopt::Std;
use File::Basename;
$| = 1;

### Check for debug flag
getopts('d');  # If debug flag is set, $opt_d == 1
  
# Save current directory but run script from PVT home directory
$current_directory = getcwd();
$home_pvt = dirname($0);
chdir $home_pvt or die "Can\'t run from the PVT home directory\n";

### Create main window
$mw = MainWindow->new();
$mw->title("PDS Validation Tools Version 1.0");

# Help text for path selection
$explain_path_selection = <<"EOF1";
Enter the path to the root directory on the volume you want to
validate. Alternatively, you can press the 'BROWSE...' button
and open any file on your volume's root directory.
EOF1

# Help text for test selection
$explain_test_selection = <<"EOF2";
Select any of the following tests to run. The 'Choose/Clear All'
button will select all the tests (unless all the tests are
already selected, in which case it will unselect all the tests).
EOF2

### Frame 1 is for obtaining the volume root

$frame1 = $mw->Frame(
                     -borderwidth => 2,
                     -relief => 'groove'
                    )->pack(
                    	    -anchor => 'nw',
                            -padx => 10,
                            -pady => 5
                            );

$frame1->Label(
               -font => "Helvetica 18 normal",
               -text => "Set Path to Volume Root"
              )->pack(
                      -anchor => 'w'
                     );
                     
$frame1->Label(
               -text => "$explain_path_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );

$frame1->Entry(
               -background => 'white',
               -textvariable => \$root_dir
              )->pack(
                      -side => 'left',
                      -padx => 5
                     );
              
$root_dir = "e:\\"; # Default root directory

$frame1->Button(
                -text => "Browse...",
                -command => \&get_root_dir
               )->pack(
                       -side => 'left',
                       -pady => 5
                     );

### Frame 2 is for choosing the tests to run

$frame2 = $mw->Frame(
                     -borderwidth => 2,
                     -relief => 'groove'
                    )->pack(
                            -anchor => 'w',
                            -padx => 10,
                            -pady => 5,
                            -expand => 1,
                            -fill => 'x'
                           );

$frame2->Label(
               -font => "Helvetica 18 normal",
               -text => "Choose Tests to Run",
              )->pack(
                      -anchor => 'w'
                     );
                 
$frame2->Label(
               -text => "$explain_test_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );

$frame2->Button(
                -text => "Choose/Clear All",
                -command => \&choose_all,
                )->pack(
                        -anchor => 'w',
                        -padx => 5
                       );


if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {

    $frame2->Checkbutton(
                         -text => "Check ISO9660 compliance",
                         -variable => \$checkiso,
                         -command => \&tbox_state,
                         )->pack(
                                 -anchor => 'w'
                                );
}
else {
    $frame2->Checkbutton(
                         -text => "Check ISO9660 compliance",
                         -variable => \$checkiso
                        )->pack(
                                -anchor => 'w'
                               );
}

if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {

     $frame2->Label(
                    -text => "Enter the path to the cdrom device below: (i.e. /dev/cdrom)",
                    -justify => 'left'
                   )->pack(
                           -anchor => 'w'
                          );

     $tbox = $frame2->Entry(
                    -background => 'light gray',
                    -textvariable => \$cd_dir,
                    -state => 'disabled'
                    )->pack(
                            -anchor => 'w',
                            -padx => 5
                           );
}

$frame2->Checkbutton(
                     -text => "Check for required files",
                     -variable => \$checkreq
                    )->pack(
                            -anchor => 'w'
                           );

# 04-21-03 MDC - This feature is built in to the lvtool already.
# Select the "Change label validation settings" box to enable this 
# feature.

#$frame2->Checkbutton(
#                     -text => "Check that every file has a label",
#                     -variable => \$checklab
#                    )->pack(
#                            -anchor => 'w',
#                           );

$frame2->Checkbutton(
                     -text => "List label validation errors",
                     -variable => \$listerr,
                     -command => \&settings_state
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "List new keywords",
                     -variable => \$listnew
                     )->pack(
                             -anchor => 'w'
                            );

$frame2->Checkbutton(
                     -text => "Check the index files",
                     -variable => \$checkidx
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "Test formatting in ASCII files",
                     -variable => \$testline
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "Check references",
                     -variable => \$checkref
                    )->pack(
                            -anchor => 'w'
                           );

$frame2->Checkbutton(
                     -text => "List keyword/value pairs",
                     -variable => \$listkwv
                    )->pack(
                            -anchor => 'w'
                           );

$cb = $frame2->Checkbutton(
		   -text => "Change Label Validation Settings",
		   -variable => \$settings,
		   -state => 'disabled',
		   )->pack(
		   	   -anchor => 'w',
		   	   -pady => 15
		      );
### Frame 3 run the tests or exits

$frame3 = $mw->Frame(
                     -borderwidth => 2,
                    )->pack(
                            -anchor => 'w',
                            -padx => 10,
                            -expand => 1,
                            -fill => 'x'             
                           );

$frame3->Button(
            -text => "Run Tests",
            -command => \&run_tests
               )->pack(
                   -side => 'left'
                  );

$frame3->Button(
                -text => "Exit",
                -command => sub { exit }
                )->pack(
                        -side => 'right'
                       );

MainLoop;

###########################
### Support Subroutines ###
###########################

# Get the root directory through a file-browsing dialog
sub get_root_dir {

    $file = $frame1->getOpenFile();
    $file =~ s#\/[^/]+$#\/#; # Get directory from filename
    $file = lc $file if ($file =~ m/\w:/); # Another evil Windows hack
    $root_dir = $file;
}

# Choose or clear all the checkboxes
sub choose_all {
	
    # Get the sum of all the tests selected
    $first5 = $checkiso + $checkreq + $checklab + $listerr + $listnew;
    $last4 = $checkidx + $testline + $checkref + $listkwv;
    $all = 0 + $first5 + $last4;
    
    # If all the tests are selected, clear them
    if ($all == 9) {
        $checkiso = 0;
        $checkreq = 0;
        $checklab = 0;
        $listerr = 0;
        $listnew = 0;
        $checkidx = 0;
        $testline = 0;
        $checkref = 0;
        $listkwv = 0;
    # Otherwise, select all the tests
    } else {
        $checkiso = 1;
        $checkreq = 1;
        $checklab = 1;
        $listerr = 1;
        $listnew = 1;
        $checkidx = 1;
        $testline = 1;
        $checkref = 1;
        $listkwv = 1;
    }
    
    settings_state();
    
    if( ($^O =~ m/solaris/) || ($^O =~ m/linux/) ) {
    	tbox_state();
    }
}

# Run the validation tests
sub run_tests {
	
    # Get the sum of all the tests selected
    $first5 = $checkiso + $checkreq + $checklab + $listerr + $listnew;
    $last4 = $checkidx + $testline + $checkref + $listkwv;
    $all = 0 + $first5 + $last4;
    
    # If the volume root directory can't be found, give error dialog
    unless (-d $root_dir) {
        &error_dialog("Can\'t open volume root: $root_dir");
        return;
    }
    
    # If no tests are specified, give error dialog
    if ($all == 0) {
        &error_dialog("No tests chosen");
        return;
    }
    
    # If validate.pl isn't present, give error dialog
    unless (-e "validate.pl") {
        &error_dialog("Can\'t find validate.pl");
    }
    
    if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {
    	if( ($checkiso == 1) && ($cd_dir eq "") ) {
    	   &error_dialog("Path to the cdrom device not specified");
    	   return;
    	}
    }
        
    # 04-21-03 MDC - Since we've taken out the checklab option,
    # bump up the numbers of the commands that follow. 
    # i.e. $listerr was option 4, now it is option 3.
    
    # Build the command string and run it
    my $command = "perl validate.pl -";
    $command = $command . "1" if ($checkiso == 1);
    $command = $command . "2" if ($checkreq == 1);
 #  $command = $command . "3" if ($checklab == 1);
    $command = $command . "3" if ($listerr == 1);
    $command = $command . "4" if ($listnew == 1);
    $command = $command . "5" if ($checkidx == 1);
    $command = $command . "6" if ($testline == 1);
    $command = $command . "7" if ($checkref == 1);
    $command = $command . "8" if ($listkwv == 1);
    $command = "perl validate.pl -b" if ($all == 9);
    $command = $command . " -s" if ( ($listerr ==1) && ($settings == 1) );
    $command = $command . " " . "-c" . " " . $cd_dir if ( ($checkiso == 1) && (($^O =~ m/linux/) || ($^O =~ m/solaris/)) );
    $command = $command . " " . $root_dir . " " . $current_directory;
    print "Command = $command\n" if ($opt_d);
    system("$command") unless ($opt_d);
}

# Create error dialog
sub error_dialog {

    my $message = shift @_;
    
    $mw->Dialog(
                -title => 'ERROR',
                -text => "$message",
                -bitmap => 'error',
                -default_button => 'OK',
                -buttons        => ['OK']
               )->Show();
               
}

# 04-29-03 MDC
# Toggles the state of the "Change Label Validation Settings" button 
# based on if the "List Label Validation Errors" test is selected
sub settings_state {
   if($listerr) {
   	$cb->configure(-state => 'normal');
   }
   else {
      $cb->configure(-state => 'disabled');
   }
}


sub tbox_state {
    if($checkiso) {
         $tbox->configure(-background => 'white',
			  -state => 'normal'
			  );
     }
    else {
	$tbox->configure(-background => 'light gray',
			 -state => 'disabled'
			);
	$cd_dir="";
    }
}
