#!/usr/bin/perl
#
# Script: validate.pl - Runs validation tests on a volume
# Usage:  validate.pl [-b] [-123456789] [-n] [-s] <volume root> [report directory]
#
# Author: Joel Wilf
# Date: 7/29/99
#
# Description:
#   This script can run any or all of the following validation tests.
#   (Note that any of these tests can be run individually, using the
#   programs shown in parentheses, below:)
#
#     1) check ISO9660 compliance (isocheck, checkiso.pl)
#     2) check for required files and directories (checkreq.pl)
#     3) check whether every file has a label (checklab.pl)
#     4) find label validation errors (lvtool, listerr.pl)
#     5) find new keywords and standard values (lvtool, listnew.pl)
#     6) check that the index file points to the data (checkidx.pl)
#     7) test for errors in ASCII files (testline.pl)
#     8) check references against reference database (checkref.pl)
#     9) list keyword/value pairs for inspection (kwvtool, listkwv.pl)
#
#   There are two ways to run validate.pl. The first is interactively,
#   where the only input is the root directory of the volume to be
#   validated. The script prompts the user, asking which tests to
#   perform.
#
#   The script can also be run in "batch mode," by setting the -b
#   option. If -b is set, all tests are run automatically, no
#   questions asked. If -<some numbers> is set (and -b is not), then
#   the tests corresponding to the numbers given are run.  For
#   example, -b -1357 runs the tests numbered 1, 3, 5, and 7, as
#   defined above.
#
#   An additional option was added to account for the "pseudo PDS"
#   attached labels used by NAIF. If processing NAIF volumes, use
#   this -n flag.
#
#   The volume root (the path to the root directory of the volume) is
#   required. The test report will have the name "<VOLUME_ID.rpt>" and
#   will be placed in the current working directory, unless the 
#   [report directory] is specified.
# End:
#
# Revisions:
#   1999-08-29, J. Wilf, Added option to select tests in batch mode
#   1999-10-02, J. Wilf, Added option to specify report directory
#   2000-03-31, J. Wilf, Added option for NAIF special case
#   2000-04-03, J. Wilf, Added ISOCHECK and KWVTOOL checks
#   2002-03-12, J. Wilf, Changed default directory to current one
#   2003-03-12, M. Cayanan, Added another option, -s, to allow user to
#		configure the label validation error test. 
#   2003-04-15, M. Cayanan, Added a GUI display that allows users to 
#               configure the label validation error test, exclude 
#               warnings, info, and error messages to the users liking.
#   2003-04-17, M. Cayanan, Removed any code dealing with checklab.pl since
#               lvtool does a better job of checking for files with labels. 
# End:

use Tk;
use Tk::LabEntry;
use Cwd;
use Getopt::Std;
use File::Find;
use File::Basename;
$| = 1;

# Save current directory but run script from PVT home directory
$current_directory = getcwd();
$home_pvt = dirname($0);
chdir $home_pvt or die "Can\'t run from the PVT home directory\n";

# Get options and root directory

if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {
   $usage = "validate.pl [-b] [-12345678] [-n] [-s] [-c] <cd-rom device> <volume root> [report directory]";
   getopts('b12345678nsc'); # If -b is set, $opt_b == 1, etc.
}
else {
   $usage = "validate.pl [-b] [-12345678] [-n] [-s] <volume root> [report directory]";
   getopts('b12345678ns'); # If -b is set, $opt_b == 1, etc.
}

$naif_option = "";	# No NAIF option unless explicitly set

if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {
   if($opt_c == 1) {
      $cd_dir = shift or die "$usage\n";
   }
}

$root_dir = shift or die "$usage\n";
die "Can\'t find $root_dir\n" unless (-e $root_dir);
die "$root_dir isn't a directory\n" unless (-d $root_dir);

# Get VOLDESC.CAT (ignoring case)
$voldesc = "$root_dir/voldesc.cat" if (-e "$root_dir/voldesc.cat");
$voldesc = "$root_dir/VOLDESC.CAT" if (-e "$root_dir/VOLDESC.CAT");

# Use VOLDESC.CAT to get the VOLUME ID
if ($voldesc) {
    open (VOLDESC,"<$voldesc") or die "Can\'t open $voldesc\n";
    while ($line = <VOLDESC>) {
        if ($line =~ m/VOLUME_ID\s+=\s+\"*(\w+)/) {
            $volume_id = $1;
            last;
        }
    }
    close VOLDESC;
}
unless ($volume_id) {
    $volume_id = "NO_VOLUME_ID";
    print "ERROR: There should be a VOLDESC.CAT with a VOLUME ID\n\n";
}

# Set up output reports
$report = "$volume_id" . '.rpt';
$kwv_report = 'kwv-' . "$report";
$iso_report = 'iso-' . "$report";
$report_dir = shift or ($report_dir = $current_directory);
$report = $report_dir . "/" . $report if ($report_dir);
$kwv_report = $report_dir . "/" . $kwv_report if ($report_dir);
$iso_report = $report_dir . "/" . $iso_report if ($report_dir);

# For windows, replace '/' with '\'
if( ($^O !~ m/solaris/) && ($^O !~ m/linux/) ) {
   if($root_dir =~ m/\//) {
	$root_dir =~ s/\//\\/g;
   }
   $check = $root_dir;
   $last_char = chop($check);
   if( ($last_char eq "/") || ($last_char eq "\\") ) {
	chop($root_dir);
   }
}

# Added by MDC 04-01-03
# Set up command line for label validation test a.k.a. lvtool

# Command line if running in a solaris or linux environment
if( ($^O =~ m/solaris/) || ($^O =~ m/linux/) ) {
	$command = "./lvtool -a -v -nw -b3 $root_dir -d ../data/pdsdd.idx -r $current_directory/temp";
}
else { #Command line for windows platform
	$command = "bin/lvtool -a -v -nw -b3 $root_dir -d data/pdsdd.idx -r temp -f $root_dir\\*.*";
}
# Open report file and announce presence

open (REPORT,">$report") or die "Can\'t open report file\n";
print REPORT "***********************************\n";
print REPORT "* Validation Report for $volume_id *\n";
print REPORT "***********************************\n\n";
close REPORT;  # So system calls can add to it
print "\nValidating $volume_id:\n\n";

# Find out which tests to run
$ask_me = 1;
if ($opt_b or $opt_1) {
    $run_iso = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_2) {
    $run_req = 1;
    $ask_me = 0;
}
# 04-17-03 MDC - Removed this option since this check
# is built into lvtool already. This would bump up the
# option numbers below.

#if ($opt_b or $opt_3) {
#    $run_lab = 1;
#    $ask_me = 0;
#}

if ($opt_b or $opt_3) {
    $run_err = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_4) {
    $run_new = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_5) {
    $run_idx = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_6) {
    $run_asc = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_7) {
    $run_ref = 1;
    $ask_me = 0;
}
if ($opt_b or $opt_8) {
    $run_kwv = 1;
    $ask_me = 0;
}

# If no options are set, find out interactively
if ($ask_me == 1) {
    $help = '(Enter "y" for yes, "n" for no, "q" to quit): ';
    $run_all = &ask ("Run all tests? $help");
    if ($run_all == 0) {
        $run_iso = &ask ("Check ISO9660 compliance? ");
        if($run_iso) {
            if(($^O =~ m/linux/) || ($^O =~ m/solaris/)) {
        	print STDOUT "Enter the cd-rom device path: ";
        	$cd_dir = <STDIN>;
            }
        }
        $run_req = &ask ("Check for required files and directories? ");
 #      $run_lab = &ask ("Check whether every file has a label? ");
        $run_err = &ask ("Find label validation errors? ");
        if($run_err) {
           $opt_s = &ask ("Change label validation settings? ");
        }
        $run_new = &ask ("Find new keywords and standard values? ");
        $run_idx = &ask ("Check that the index file points to the data? ");
        $run_asc = &ask ("Test for errors in ASCII files?");
        $run_ref = &ask ("Check references against reference database? ");
        $run_kwv = &ask ("List keyword/value pairs for inspection? ");
    } else {
        print "\nRunning all tests...\n\n";
        $run_iso = 1;
        $run_req = 1;
  #     $run_lab = 1;
        $run_err = 1;
        $opt_s = 1;
        $run_new = 1;
        $run_idx = 1;
        $run_asc = 1;
        $run_ref = 1;
        $run_kwv = 1;
    }
}

# 04-03-03 MDC
# If the settings flag was set, then show GUI to allow user to change the
# label validation settings.
if ($run_err == 1 && $opt_s == 1) {
    &display_settings;
}	

# Test 1 - Check ISO9660 compliance (isocheck, checkiso.pl)
if ($run_iso == 1) {
    print "Checking ISO9660 compliance...\n\n";

    if( ($^O =~ m/linux/) || ($^O =~ m/solaris/) ) {
	system("perl checkiso.pl $cd_dir $iso_report >> $report");
    }
    else {
	system ("perl checkiso.pl $root_dir $iso_report >> $report");
    }
}

# Test 2 - Check for required files and directories (checkreq.pl)
if ($run_req == 1) {
    print "Checking for required files and directories...\n\n";
    system ("perl checkreq.pl $root_dir >> $report");
}


# 04-17-03 MDC - Removed code of checklab.pl since lvtool
# does this check for us too

# Test 3 - Check whether every file has a label
### Note: some of this code can be removed when lvtool checks for labels.
#$ran_checklab = 0;
# if ($run_lab == 1) {
#    print "Checking whether every file has a label...\n\n";
#    $naif_option = "-d" if ($opt_n == 1);  # Skip NAIF attached labels
#    system ("perl checklab.pl $naif_option $root_dir >> $report");
#    $label_list = "label.lst";
#     $ran_checklab = 1;
# }

# To find validation errors or new keywords, run lvtool on every label
# Note: this section will be revised when lvtool checks for labels
if ($opt_b or $run_all or $run_err or $run_new) {
    print "Running lvtool on every label... \n\n";

# 04-17-03 MDC - Removed     
#     if ($ran_checklab == 0) {
#     	$temp = "temp";
#        system ("perl -d:ptkdb checklab.pl $root_dir > $temp");
#        unlink $temp;
#    }

#    open (LABEL, "<label.lst");
    open (LVREPORT, ">lv.rpt");
#     @all_labels = <LABEL>;
#     foreach $label (@all_labels) {
#      Getting strange messages when expanding sturcture pointers (-e flag)
#       system ("bin/lvtool -a -e -v -nw -b3 $root_dir -d data/pdsdd.idx -r temp -f $label");
#       system ("bin/lvtool -a -v -nw -b3 $root_dir -d data/pdsdd.idx -r temp -f $label");
	
	if( ($^O =~ m/solaris/) || ($^O =~ m/linux/) ) {
	        # MDC - Have to switch to the bin directory. For some reason,
	        # it is only when you're in the bin directory that lvtool can recursively
	        # look inside the directories.
		chdir "$current_directory/bin" or die "Can't cd to $current_directory/bin: $!\n";
		system ("$command -f $root_dir/\"*.*\"");
		chdir "$current_directory" or die "Can't cd to $current_directory: $!\n";
	}
	else {
		# system ("$command -f $root_dir/*.*");
		system ("$command");
	}
#	system ("$command -fin label.lst");
        open (TEMP, "<temp");
        while (<TEMP>) {
            print LVREPORT;
        }
        close TEMP;
#    }
#    close LABEL;
    close LVREPORT;
}

# Test 4 - Find label validation errors (listerr.pl)
if ($run_err == 1) {
    print "\nFinding label validation errors...\n\n";
    
    # If the omit file exists, then include that in the command line
    # when running listerr.pl
    if(($opt_s) && (-e $logfile) ) {
    	system ("perl listerr.pl -e $logfile lv.rpt >> $report");
    }
    else {
    	system ("perl listerr.pl lv.rpt >> $report");
    } 	
}

# Test 5 - Find new keywords and standard values (listnew.pl)
if ($run_new == 1) {
    print "Finding new keywords and standard values...\n\n";
    system ("perl listnew.pl lv.rpt >> $report");
}

# Test 6 - Check that the index file points to the data (checkidx.pl)
if ($run_idx == 1) {
    print "Checking that the index file points to the data...\n\n";
    system ("perl checkidx.pl $root_dir >> $report");
}

# Test 7 - Test text files for standards compliance (testline.pl)
if ($run_asc == 1) {
    print "Test for errors in ASCII files...\n\n";
    ### To skip attached labels, remove the -l flag
    $naif_option = "-s" if ($opt_n == 1);  # Skip NAIF attached labels
    system ("perl testline.pl $naif_option $root_dir >> $report");
}

# Test 8 - Test references against reference database (checkref.pl)
if ($run_ref == 1) {
    print "Testing references against the reference database...\n\n";
    system ("perl checkref.pl $root_dir >> $report");
}

# Test 9 - List keyword/value pairs for inspection (kwvtool, listkwv.pl)
if ($run_kwv == 1) {
    print "Listing keyword/value pairs for inspection...\n\n";
    system ("perl listkwv.pl -u $root_dir $kwv_report");
}

# Cleanup temp files
unlink "temp";
###unlink "label.lst";
unlink "lv.rpt";

# Tell everyone we're done
open (REPORT,">>$report");
print REPORT "See kwv.rpt for keyword/value pairs.\n" if ($run_ref == 1);
print REPORT "\n\n*******************************\n";
print REPORT "* End of report for $volume_id *\n";
print REPORT "*******************************\n\n";
close REPORT;
# Ring the bell
#print "\a\a\a\a\a";
print "\nTests complete. The report is in: $report\n";


#######################
# Support Subroutines #
#######################

# Subroutine to ask questions and get answers
sub ask {
	
    my $question = shift @_;
    
    print "$question ";
    $answer = <STDIN>;
    exit if ($answer =~ m/q/i);
    if ($answer =~ m/y/i) {
        return 1;
    } else {
        return 0;
    }
}

sub display_settings {
	
   ### Create main window
   $mw = MainWindow->new();
   $mw->title("Label Validation Settings");
    
   $logfile = "logfile";
 
$explain_error_selection = <<"EOF1";
Select the types of error messages that you would like the label validation 
test to ignore. Holding down the cntl key while clicking on the 
messages will allow you to make multiple selections.
EOF1

$explain_warning_selection = <<"EOF2";
Select the types of warning messages that you would like the label validation test to ignore. 
Holding down the cntl key while clicking on the messages will allow you to make multiple selections. 
EOF2
   
   # Display listbox with warning messages
   
   $frame2 = $mw->Frame(
                     -borderwidth => 4,
                     -relief => 'groove'
                    )->pack(
                            -padx => 5,
                            -pady => 5,
                            -side => 'top',
                            -fill => 'both'
                           );


   # Frame 1 to display list box with error messages
    
   $frame1 = $mw->Frame(
                     -borderwidth => 4,
                     -relief => 'groove',
                    )->pack(
                            -padx => 5,
                            -pady => 5,
                            -side => 'left',
                            -fill => 'both'
                           );  
  
  
    ### Frame 3 to modify lvtool output options

   $frame3 = $mw->Frame(
                     -borderwidth => 2,
                     -relief => 'sunken'
                        )->pack(
                            -padx => 10,
                            -pady => 10,
                            -anchor => 'e',
                            -fill => 'both'
                           );
    
   $frame1->Label(
               -font => "Helvetica 18 normal",
               -text => "Possible Error Messages"
              )->pack(
                      -anchor => 'w'
                     );
                         
   $frame1->Label(
               -text => "$explain_error_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );
   
   
   $lb1 = $frame1->Scrolled( "Listbox", 
   			-scrollbars => "e", 
   			-height => 15, 
  			-selectmode => "extended",
  			-exportselection => 0,
  			-width => 0,
  			-relief => 'sunken',
   			-background => 'white',
   			-highlightcolor => 'blue',
   			-yscrollcommand => ['set' => $yscrollbar],
   		      )->pack(
   		               -side => 'left'
   		               # -fill => 'both'
   		             );
      
   $frame2->Label(
               -font => "Helvetica 18 normal",
               -text => "Possible Warning Messages"
              )->pack(
                      -anchor => 'w'
                     );
                     
   $frame2->Label(
               -text => "$explain_warning_selection",
               -justify => 'left'
              )->pack(
                      -anchor => 'w'
                     );
   
   
   $lb2 = $frame2->Scrolled( "Listbox", 
   			-scrollbars => "e", 
   			-height => 15,
  			-selectmode => "extended",
  			-exportselection => 0,
  			-width => 0,
  			-relief => 'sunken',
   			-background => 'white',
   			-highlightcolor => 'blue',
   			-yscrollcommand => ['set' => $yscrollbar],
   		      )->pack(
   		               -side => 'left',
   		               # -fill => 'both'
   		             );
   
   # Initialize Entry widget with default value of '300' max errors
   $max_err = 300;
   $frame3->LabEntry(-label => "Max Errors Allowed:", 
   		     -textvariable => \$max_err,
   		     -background => 'white',
   		     -labelPack => [ -side => 'left', -anchor => 'w' ]
   		     )->pack(
   		     	     -pady => 15,
   		     	     -side => 'top',
   		     	     -anchor => 'nw');
   
   $frame3->Checkbutton(
                     -text => "Ignore All Warning Messages",
                     -variable => \$no_warnings
                    )->pack(
                            -anchor => 'w',
                            -pady => 15
                           );
                           
   $frame3->Checkbutton(
   	 	     -text => "Ignore Info Messages",
   	 	     -variable => \$no_info
   	            )->pack(
   	            	    -anchor => 'w',
   	            	    -pady => 15
   	            	    );
   
   $l3d = 1;
   $frame3->Checkbutton(
   		     -text => "Validate Only Files That Contain Level 3 Labels",
   		     -variable => \$l3d
   		    )->pack(
   		    	   -anchor => 'w',
   		    	   -pady => 15
   		    	   );
   	            	    
   $frame3->Checkbutton(
   		     -text => "Retrieve/Clear Saved Messages",
   		     -variable => \$clear_logfile,
   		     -command => sub {
   		        if($clear_logfile) {
   		           clear();
   		        }
   		        else {
   		           retrieve_saved_msgs();
   		        }
   		      }
   		     )->pack(
   		     	    -anchor => 'w',
   		     	    -pady => 15
   		     	    );
   
   # Used for the "Done" button
   
   $frame4 = $mw->Frame(
   		      -borderwidth => 4
   		       )->pack(
   		       	      -side => 'bottom',
   		       	      -anchor => 's'
   		       	      );
   
   $frame4->Button(
   		  -text => "Done",
   		  -relief => 'raised',
   		  -padx => 10,
   		  -command => \&configure 
   		  )->pack(
   		  	 -pady => 5,
   		  	 -padx => 15,
   		  	 -side => 'left'
   		  	 );
   
   $frame4->Button(
   		 -text => "Cancel",
   		 -relief => 'raised',
   		 -padx => 10,
   		 -command => sub{ $mw->destroy(); }
   		 )->pack(
   		 	-pady => 5,
   		 	-padx => 15,
   		 	-side => 'right'
   		 	);
     
   # Grab the messages in message_list.txt       
   open(MSGFILE, "message_list.txt") || die("can't open message_list.txt file"); 
   
   #First, grab the error message list from the file
   while( chop($line = <MSGFILE>) && ($line !~ /END OF ERROR LIST/) )
   {
   	if($line !~ /POSSIBLE ERROR MESSAGES/)
   	{
   		@errmsgs = split /=\s+/, $line;
   		
   		#display_error_list is the list that the user sees in the GUI
   		
   		#real_error_list is the list that will be used to do the filtering
   		#of unwanted messages
   		push @display_error_list, $errmsgs[0];
   		push @real_error_list, $errmsgs[1];
   	}
   }
   
   #Next, grab the warning message list from the file
   #Skip blank lines first and header for the warnings list
   while( ($line = <MSGFILE>)  && ($line !~ /END OF WARNING LIST/) )
   {
   	#skip blank lines
   	if($line eq "\n") {
   	    next;
   	}
   	
   	chop($line);
   	
   	if($line !~ /POSSIBLE WARNING MESSAGES/)
   	{
   	     @warnmsgs = split /=\s+/, $line;
   	     

   	     push @display_warning_list, $warnmsgs[0];
   	     push @real_warning_list, $warnmsgs[1];
   	}
   }
   
   # insert error and warning lists onto the listbox
   $lb1->insert(0, @display_error_list);  
   $lb2->insert(0, @display_warning_list);
   
   close MSGFILE;
   
   # Check the logfile to see if any messages were saved
   # from a previous run of this tool. If there was, then
   # this will automatically select those messages that the
   # user did not want to see
   
   retrieve_saved_msgs();   
   		
   MainLoop;
   
}   

sub configure {
    
    # Enter in more command line options to label validation test
    # if they were selected
    
    $command = $command . " -l3d" if($l3d);  
    $command = $command . " -nwarn" if($no_warnings);
    $command = $command . " -ninfo" if($no_info);
    $command = $command . " -max " . "$max_err" if($max_err =~ /\d/);


    if($clear_logfile) {
    	unlink $logfile;
    }
    
    #Retrieve the error and warning messages the user selected to ignore
    
    @ignore_errs = $lb1->curselection();
    @ignore_warns = $lb2->curselection();
    
    if( ((@ignore_errs + 0) != 0) || ((@ignore_warns + 0) != 0) ) {
       open(LOGFILE, ">$logfile") or die "Can't open logfile";
       
       foreach(@ignore_errs) {
    	  print LOGFILE "$real_error_list[$_]\n";
       }
    
       foreach(@ignore_warns) {
    	  print LOGFILE "$real_warning_list[$_]\n"; 
       }
    
       close LOGFILE;
    }
    else {
    	unlink $logfile;
    }
    
    $mw->destroy();
}

sub clear() {
	
    if($clear_logfile) {
    	$lb1->selectionClear(0, "end");
    	$lb2->selectionClear(0, "end");
    }

}

# Look at the logfile and retrieve any messages that were
# saved. The messages that were saved will be highlighted
# in the settings screen.

sub retrieve_saved_msgs() {
	
     if( open(SAVEDMSGS, "<$logfile") ) {
   	while( $line = <SAVEDMSGS> ) {
   		
   		chop ($line);
   		$counter = 0;
   		foreach $msg (@real_error_list) {
   	 	    if( $line eq $msg ) {
   			$lb1->selectionSet($counter);
   			last;
   		    }
   		    ++$counter;
   		}
   		
   		$counter = 0;
   		foreach $msg (@real_warning_list) {
   		    if( $line eq $msg ) {
   		    	$lb2->selectionSet($counter);
   		    	last;
   		    }
   		    ++$counter;
   		}
   		
   	}
   	close SAVEDMSGS;
    }
}
